package it.unibas.mastermind.controllo;

import it.unibas.mastermind.modello.Combinazione;
import it.unibas.mastermind.modello.Partita;
import it.unibas.mastermind.modello.Risposta;
import it.unibas.utilita.Console;
import it.unibas.utilita.Logger;

public class Principale {
    
    private void esegui() {
        boolean continua = true;
        while (continua) {
            int scelta = schermoMenuPrincipale();
            if (scelta == 1) {
                giocaPartita();
            } else if (scelta == 2) {
                continua = false;
            }
        }
    }
    
    private void giocaPartita() {
        Partita partita = new Partita();
        System.out.println("");
        System.out.println("Devi indovinare la combinazione nascosta. Immetti i tuoi tentativi");
        System.out.print("(devono essere di " + Combinazione.CIFRECOMBINAZIONE + " cifre diverse ");
        System.out.println(" comprese tra 1 e " + Combinazione.NUMEROCIFRE + ")");
        boolean esci = false;
        while (!partita.isConclusa() && !esci) {
            Combinazione tentativo = schermoLeggiTentativo();
            Risposta risposta = partita.verificaTentativo(tentativo);
            Logger.logFine("Principale", "giocaPartita", "combinazione: " + partita.getCombinazione() + " - tentativo: " + tentativo + " - risposta: " + risposta);
            schermoVisualizzaTentativi(partita);
            if (partita.isConclusa()) {
                System.out.println("Complimenti. Hai indovinato");
            } else {
                esci = schermoProsegui();
            }
        }
        if (esci) {
            System.out.println();
            System.out.println("Peccato. La combinazione era: " + partita.getCombinazione());
            System.out.println();
        }
    }
    
    private boolean schermoProsegui() {
        System.out.println("1. Prosegui - 2. Abbandona");
        System.out.print("Scelta -> ");
        int scelta = Console.leggiIntero();
        while ((scelta < 1 || scelta > 2)) {
            System.out.println("Errore. Scelta scorretta.");
            System.out.print("Riprova -> ");
            scelta = Console.leggiIntero();
        }
        if (scelta == 2) {
            return true;
        }
        return false;
    }
    
    private void schermoVisualizzaTentativi(Partita partita) {
        if (partita.numeroTentativi() != 0) {
            System.out.println();
            System.out.println("Tentativi effettuati");
            System.out.println("--------------------");
            for (int i = 0; i < partita.numeroTentativi(); i++) {
                System.out.println(partita.getTentativo(i) + " " + partita.getRisposta(i));
            }
            System.out.println();
        }
    }
    
    private Combinazione schermoLeggiTentativo() {
        System.out.println("");
        System.out.print("Immetti il tuo tentativo -> ");
        String tentativo = Console.leggiStringa();
        Combinazione combinazione = estraiCombinazione(tentativo);
        while (combinazione == null) {
            System.out.println("Errore. Scelta scorretta.");
            System.out.println("Riprova -> ");
            tentativo = Console.leggiStringa();
            combinazione = estraiCombinazione(tentativo);            
        }
        return combinazione;
    }
    
    private Combinazione estraiCombinazione(String tentativo) {
        if (tentativo.length() != Combinazione.CIFRECOMBINAZIONE) {
            return null;
        } 
        char[] caratteri = tentativo.toCharArray();
        Combinazione combinazione = new Combinazione();
        for (int i = 0; i < Combinazione.CIFRECOMBINAZIONE; i++) {
            combinazione.setCifra(i, (int)caratteri[i] - 48);
        }
        if (!Combinazione.verificaCombinazione(combinazione)) {
            return null;
        }
        return combinazione;
    }
    
    private int schermoMenuPrincipale() {
        System.out.println("--------------------------------");
        System.out.println("         Master Mind");
        System.out.println("--------------------------------");
        System.out.println();
        System.out.println(" 1. Nuova partita");
        System.out.println(" 2. Esci");
        System.out.println();
        System.out.print("Scelta -> ");
        int scelta = Console.leggiIntero();
        while ((scelta < 1 || scelta > 2)) {
            System.out.println("Errore. Scelta scorretta.");
            System.out.print("Riprova -> ");
            scelta = Console.leggiIntero();
        }
        return scelta;
    } 
   
    public static void main(String[] args) {
        Principale p = new Principale();
        p.esegui();
    }
}
