package it.unibas.mastermind.modello;

public class Combinazione {
    
    public static final int NUMEROCIFRE = 6;
    public static final int CIFRECOMBINAZIONE = 4;
    
    private static java.util.Random generatore = new java.util.Random();
    
    private int[] cifre = new int[CIFRECOMBINAZIONE];
    
    public static boolean verificaCombinazione(Combinazione combinazione) {
        for (int i = 0; i < CIFRECOMBINAZIONE; i++) {
            if (combinazione.getCifra(i) < 1 || combinazione.getCifra(i) > NUMEROCIFRE) {
                return false;
            }
        }
        return combinazione.checkDiverse();
    }
    
    public void genera() {
        for (int i = 0; i < CIFRECOMBINAZIONE; i++) {
            generaCifra(i);
        }
    }
    
    public int getCifra(int i) {
        assert(0 <= i && i <= CIFRECOMBINAZIONE) : "Cifra inesistente";
        return cifre[i];
    }
    
    public void setCifra(int i, int valore) {
        assert(0 <= i && i <= CIFRECOMBINAZIONE) : "Cifra inesistente";
        assert(0 <= valore && valore <= NUMEROCIFRE) : "Valore scorretto";
        assert(checkDiverse()) : "Cifra gia' presente";
        cifre[i] = valore;
    }

    private void generaCifra(int posizione) {
        while (true) {
            cifre[posizione] = generatore.nextInt(NUMEROCIFRE) + 1;
            if (checkDiverse()) {
                return;
            }
        }
    }
    
    private boolean checkDiverse() {
        for (int i = 0; i < Combinazione.CIFRECOMBINAZIONE; i++) {
            if (cifre[i] != 0 && contaOccorrenze(cifre[i]) != 1) {
                return false;
            }
        }
        return true;
    }
    
    private int contaOccorrenze(int cifra) {
        int conta = 0;
        for (int i = 0; i < CIFRECOMBINAZIONE; i++) {
            if (cifre[i] == cifra) {
                conta++;
            }
        }
        return conta;
    }
    
    public String toString() {
        String risultato = "";
        for (int i = 0; i < CIFRECOMBINAZIONE; i++) {
            risultato += cifre[i] + " ";
        }
        return risultato;
    }
    
}
